"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.QuerySchemaPipe = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const ajv_1 = require("ajv");
const helper_1 = require("../../../helper");
class QuerySchemaPipe {
    constructor(repository, ajvCall) {
        this.repository = repository;
        this.ajvCall = ajvCall;
        const schemaName = (0, helper_1.getEntityName)(this.repository.target);
        this.validateFunction = this.ajvCall.getSchema(`inputQuerySchema-${schemaName}`);
    }
    static inject(pip) {
        (0, common_1.Inject)(ajv_1.default)(pip, 'ajvCall', 1);
    }
    getErrorMsg(field) {
        switch (field) {
            case 'fields': {
                const fieldArray = Object.keys(this.validateFunction.schema['properties']['fields']['properties']);
                return `Incorrect props in fields object of query. For resource "${(0, helper_1.getEntityName)(this.repository.target)}" allow "fields" of query params "${fieldArray.join(',')}"`;
            }
            case 'filter': {
                return `Incorrect object filter in query`;
            }
            case 'include': {
                return `Should be list field by comma`;
            }
            case 'sort': {
                return `Should be list field by comma`;
            }
            case 'page': {
                return `Incorrect object page in query`;
            }
        }
    }
    transform(value) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const validate = this.validateFunction(value);
            if (validate) {
                return value;
            }
            const errors = {};
            for (const error of this.validateFunction.errors) {
                const parameterParts = error.instancePath
                    .split('/')
                    .filter((value) => value !== '');
                errors[parameterParts[0]] = {
                    source: { parameter: parameterParts[0] },
                    detail: this.getErrorMsg(parameterParts[0]),
                };
            }
            throw new common_1.BadRequestException(Object.values(errors));
        });
    }
}
exports.QuerySchemaPipe = QuerySchemaPipe;
//# sourceMappingURL=query-schema.pipe.js.map